#include "CommQueue.h"
#include <assert.h>

CommQueue::CommQueue(int queuesize)
{
	mSize = queuesize;
	mCurrentSize = 0;
	mHeadIndex = 0;
	mTailIndex = 0;

   canary1 = canary2 = canary3 = canary4 = 0xdeadbeef;
	assert(canary1 == 0xdeadbeef);
	assert(canary2 == 0xdeadbeef);
	assert(canary3 == 0xdeadbeef);
	assert(canary4 == 0xdeadbeef);

}

CommQueue::~CommQueue()
{
}

bool CommQueue::add(const struct CMessage* msg)
{
	assert(canary1 == 0xdeadbeef);
	assert(canary2 == 0xdeadbeef);
	assert(canary3 == 0xdeadbeef);
	assert(canary4 == 0xdeadbeef);
	
	comm_mutex.lock();
		
	while (mCurrentSize == mSize)
	{
		canWrite_cond.await(&comm_mutex, 'W');
	}
	
	assert((mTailIndex < mSize) && (mTailIndex >= 0));
	
	memcpy(&mQueue[mTailIndex], msg, sizeof(struct CMessage));
//	std::cout << mTailIndex << " ";
	mTailIndex--;
	mCurrentSize++;
//	std::cout << mCurrentSize << " ";
	
	if (mTailIndex < 0)
		mTailIndex = mSize - 1;

	if (1 == mCurrentSize)
	{
//		std::cout << "Signal ";
		canRead_cond.signal('R');
	}
	
//	std::cout << "ADD: SUCCESS: " << msg.mType;

	comm_mutex.unlock();

	return true;
}

bool CommQueue::getMessage(struct CMessage *msg)
{
	assert(canary1 == 0xdeadbeef);
	assert(canary2 == 0xdeadbeef);
	assert(canary3 == 0xdeadbeef);
	assert(canary4 == 0xdeadbeef);
	
	comm_mutex.lock();

	while (0 == mCurrentSize)
	{
		canRead_cond.await(&comm_mutex, 'R');
	}

	assert((mHeadIndex < mSize) && (mHeadIndex >= 0));

	memcpy(msg, &mQueue[mHeadIndex], sizeof(struct CMessage));
//	std::cout << mHeadIndex << " ";
	mCurrentSize--;
	mHeadIndex--;
//	std::cout << mCurrentSize << " ";

	if (mHeadIndex < 0)
		mHeadIndex = mSize -1;

	if ( (mSize-1) == mCurrentSize)
	{
//		std::cout << "Signal ";
		canWrite_cond.signal('W');
	}
	
//	std::cout << "GET: SUCCESS: " << msg.mType;

	comm_mutex.unlock();

	return true;
}

int CommQueue::getNumOfMessages(void)
{
	int size;
	assert(canary1 == 0xdeadbeef);
	assert(canary2 == 0xdeadbeef);
	assert(canary3 == 0xdeadbeef);
	assert(canary4 == 0xdeadbeef);
	
	comm_mutex.lock();
	size = mCurrentSize;
	comm_mutex.unlock();
	return size;
}

int CommQueue::getNumOfByteNeeded(void)
{
	assert(canary1 == 0xdeadbeef);
	assert(canary2 == 0xdeadbeef);
	assert(canary3 == 0xdeadbeef);
	assert(canary4 == 0xdeadbeef);
	
	return (sizeof(CommQueue) + sizeof(struct CMessage) * (mSize -1));
}

bool CommQueue::destroy()
{
	assert(canary1 == 0xdeadbeef);
	assert(canary2 == 0xdeadbeef);
	assert(canary3 == 0xdeadbeef);
	assert(canary4 == 0xdeadbeef);
	
	comm_mutex.destroy();
	canWrite_cond.destroy();
	canRead_cond.destroy();
	
	return true;
}
