#include "CommQueue.h" 
#include <iostream>
#include <fcntl.h> /* for O_ constants */
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/shm.h>
#include <sys/wait.h>
#include <ctime>
#include <signal.h>

#define SIZE 5
#define SHM_SIZE (sizeof(int) + sizeof(CommQueue) + sizeof(struct CMessage) * (SIZE-1))
#define SHM_MODE (SHM_R | SHM_W)
#define OPEN_MODUS (S_IRUSR | S_IWUSR)
#define ANZAHL_NACHRICHTEN 100000

int main ()
{
	
	pid_t child;
		key_t shm_key;
		int shmid;
		int randval;

		srand(( unsigned )time( NULL ));

		randval = rand();

		
		shm_key = ftok("./",'1');
		
		if ( (shmid = shmget(shm_key, SHM_SIZE, SHM_MODE | IPC_CREAT )) < 0)
		{
			perror("shmget");
			return 1;
		}
		
	signal(SIGCHLD, SIG_IGN);
	
	if ((child = fork()) < 0 )
	{
		perror("fork");
		return 1;
	}
	else if (child > 0) /* parent */
	{ 
		void *shm_ptr;
		int *OKID;
		CommQueue *queue;
		CMessage message;
		int j;
		srand(( unsigned )time( NULL ));

		if ( (shm_ptr = shmat(shmid, 0,0)) == (void*) - 1)
		{
			perror("shmat(parent)");
			shmctl(shmid, IPC_RMID, 0);
			return 1;
		}
		
		memset(shm_ptr, 0, SHM_SIZE);

		OKID = (int *)shm_ptr;
		shm_ptr = (int *)shm_ptr + 1;
		
		queue = new (shm_ptr) CommQueue(SIZE);
	
	   *OKID = randval;
	
		clock_t start = clock();
		for (unsigned long int i = 0; i < ANZAHL_NACHRICHTEN; i++)
		{
			j=1+(int) (10.0*rand()/(RAND_MAX+1.0));
			message.mType = j;
			if (queue->add(&message) != true)
			{
				std::cout << "ADD: Failed";
			}
			std::cout << "A " << i << std::endl << std::flush;
		}
		clock_t stop = clock();

		std::cout << "Running Time (add): " << (double) (stop - start) / CLOCKS_PER_SEC << std::endl << std::flush;
		
		wait(NULL);
		
		queue->destroy();
		shmdt(shm_ptr);
		shmctl(shmid, IPC_RMID, 0);	
	
		return 0;
	}
	else /* child */
	{
		void *shm_ptr;
		CommQueue *queue;
		CMessage message;
		
		if ( (shm_ptr = shmat(shmid, 0,0)) == (void*) - 1)
		{
			perror("shmat(child)");
			shmctl(shmid, IPC_RMID, 0);
			return 1;
		}
	
	   while (*(int *)shm_ptr != randval)
	   	usleep(10000);
	   	
	   shm_ptr = (int *)shm_ptr + 1;
	
		queue = (CommQueue*) shm_ptr;

		clock_t start = clock();
		for (unsigned long int i = 0; i < ANZAHL_NACHRICHTEN; i++)
		{
			if (queue->getMessage(&message) != true)
			{
				std::cout << "GET: FAILED";
			}
			std::cout << "G " << i << std::endl << std::flush;
		}
		clock_t stop = clock();

		std::cout << "Running Time (get): " << (double) (stop - start) / CLOCKS_PER_SEC << std::endl << std::flush;
		
		shmdt(shm_ptr);
	
		return 0;
		
	}
}

